#include <gamekit.h>
#include <avr/pgmspace.h>

/*
with the function assign_pixelfunction() it is possible to create new kindes of "active" pixel
such as blinking, up-down-dimming and others.

The way they work is not too complicated:
"normal" pixels have brightness values between 0 and 15.

If there is a value higher than 15, the driver knows that is a such a active pixel
and it calls the assigned function.
This function calculates the brightness value and returns it to the driver.
This happens while normal operation about 80 times a second and for every active pixel.

Functions that are assigned to a pixeltype have to have this structure:

uint8_t pixelfunction16(uint8_t row, uint8_t col, uint32_t systemcounter)

row and col are row and column of the pixel,
systemcounter is the systemcounter (it is increased by one every display update, about 80 times a second)

The return value of the function is the brightness value of the pixel (between 0 and 15), another active pixel value
is _not_ allowed.

A simple example:

uint8_t pixelfunction16(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  // static variables keep there value even if the function is finished; the initialisation ( the "= 0")
  // happens just at the first call
  static uint8_t val = 0;

  val++;
  if(val > gamekit_max_bright)
    val = 0;

  return val;
}

Note: if there are two or more pixels with value 16 the function is called once for every pixel
      thus val is increased multiple times each display refresh
      To avoid this check the system counter like the following example!

uint8_t pixelfunction23(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 0;
  static uint32_t old_counter = 0;

  if(systemcounter >= (old_counter + 3))
  {
    val++;
    if(val > gamekit_max_bright)
      val = 0;
    old_counter= systemcounter;
  };

  return val;
}

The actual assignment of the function to the pixeltype is done with:

gamekit.assign_pixelfunction(23, pixelfunction23);
Now the driver know that "pixelfunction16" gives the values for pixels with values of 16.


some general hints:
- the pixelfunctions has to be as fast as possible
  -> use the shortes possible datatype (e.g. uint8_t (same as unsigned char) instead of int)
  -> no floats, multiplications, randoms ect.
- values of 16 to 22 are already assigned to functions, but they can be reassigned to others
*/


uint8_t image[5][7] PROGMEM = {
  0 ,22,22,0 ,23,23,0 ,
  0 ,22,22,0 ,23,23,0 ,
  0 ,0 ,0 ,0 ,0 ,0 ,0 ,
  0 ,8 ,0 ,0 ,0 ,8 ,0 ,
  0 ,0 ,12,15,12,0 ,0 ,
};


uint8_t pixelfunction23(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 0;
  static uint32_t old_counter = 0;

  if(systemcounter >= (old_counter + 3))
  {
    val++;
    if(val > gamekit_max_bright)
      val = 0;
    old_counter= systemcounter;
  };

  return val;
}

uint8_t pixelfunction24(uint8_t row, uint8_t col, uint32_t systemcounter)
{
  static uint8_t val = 8;
  static uint32_t old_counter = 0;

  if(systemcounter >= (old_counter + 4))
  {
    val++;
    if(val > gamekit_max_bright)
      val = 0;
    old_counter= systemcounter;
  };

  return val;
}

void setup(){
  gamekit.Begin();
  
  gamekit.assign_pixelfunction(23, pixelfunction23);
  gamekit.assign_pixelfunction(24, pixelfunction24);
  
  gamekit.load_image( image );
}


void loop(){

}